import {
  intlFormatDistance,
  isSameDay,
  isSameHour,
  isSameMinute,
  parseISO,
} from "date-fns";
import {
  children,
  createMemo,
  Show,
  splitProps,
  type JSX,
  type ParentComponent,
} from "solid-js";
import useCurrentDate, { DateRefreshPercision } from "../../utils/useCurrentDate";


const TimeDistanceToNow: ParentComponent<JSX.IntrinsicElements["time"]> = (
  oprops,
) => {
  const [props, rest] = splitProps(oprops, ["datetime", "children"]);
  const child = children(() => props.children);

  const datetime = createMemo(() =>
    props.datetime ? parseISO(props.datetime) : undefined,
  );

  const now = useCurrentDate(() => {
    // This only get executed once at the start.
    // (No tracking to another reactive primitive)
    // That's usually good enough for a blog site.
    const d = datetime();
    const now0 = new Date();
    if (!d || !isSameDay(d, now0)) return DateRefreshPercision.days;
    if (isSameMinute(d, now0)) return DateRefreshPercision.seconds;
    if (isSameHour(d, now0)) return DateRefreshPercision.minutes;
    return DateRefreshPercision.hours;
  });

  const distanceText = createMemo(() => {
    return intlFormatDistance(props.datetime!, now(), {});
  });

  return (
    <Show
      when={props.datetime && isSameDay(props.datetime, now())}
      fallback={
        <time datetime={props.datetime} {...rest}>
          {child()}
        </time>
      }
    >
      <time datetime={props.datetime} {...rest}>
        {distanceText()}
      </time>
    </Show>
  );
};

export default TimeDistanceToNow;
