import type { APIRoute } from "astro";
import { experimental_AstroContainer as AstroContainer } from "astro/container";
import { loadRenderers } from "astro:container";
import { getContainerRenderer as solidContainerRenderer } from "@astrojs/solid-js";
import { getContainerRenderer as mdxContainerRenderer } from "@astrojs/mdx";
import { getCollection, render } from "astro:content";
import { x } from "xastscript";
import { postParamlink, shouldBeVisible } from "~/utils/posts";
import { toXml } from "xast-util-to-xml";

export const GET: APIRoute = async () => {
  const renderers = await loadRenderers([
    solidContainerRenderer(),
    mdxContainerRenderer(),
  ]);
  const cont = await AstroContainer.create({
    renderers: renderers,
  });
  const posts = (await getCollection("posts"))
  .filter(shouldBeVisible)
    .sort((a, b) => b.data.date.getTime() - a.data.date.getTime())
    .slice(0, 10);
  const authors = await getCollection("authors");

  const tr = x(
    "feed",
    { xmlns: "http://www.w3.org/2005/Atom" },
    x("title", { type: "text" }, "Rubicon's Rubicon"),
    x("id", {}, "https://rubicon.lightstands.xyz/"),
    x(
      "updated",
      {},
      posts.length > 0
        ? posts[0].data.date.toISOString()
        : new Date().toISOString(),
    ),
    x("link", {
      rel: "alternate",
      type: "text/html",
      hreflang: "zh",
      href: "https://rubicon.lightstands.xyz",
    }),
    x("link", {
      rel: "alternate",
      type: "application/feed+json",
      hreflang: "zh",
      href: "https://rubicon.lightstands.xyz/feed.json",
    }),
    x("link", {
      rel: "self",
      type: "application/atom+xml",
      href: "https://rubicon.lightstands.xyz/atom.xml",
    }),
    ...(await Promise.all(
      posts.map(async (item) => {
        const href = new URL(
          postParamlink(item),
          "https://rubicon.lightstands.xyz",
        ).toString();
        const { Content } = await render(item);
        return x(
          "entry",
          {},
          x("title", {}, item.data.title),
          x("id", {}, href),
          x("published", {}, item.data.date.toISOString()),
          x(
            "updated",
            {},
            item.data.updated
              ? item.data.updated.toISOString()
              : item.data.date.toISOString(),
          ),
          x("link", {
            rel: "alternate",
            type: "application/xhtml+html",
            href,
          }),
          ...authors.map((author) => {
            return x(
              "author",
              {},
              x("name", {}, author.data.name),
              (author.data.links?.length ?? 0 > 0)
                ? x("uri", {}, author.data.links![0].href)
                : undefined,
            );
          }),
          x(
            "content",
            { type: "text/html", "xml:base": href },
            await cont.renderToString(Content),
          ),
        );
      }),
    )),
  );
  return new Response(toXml(tr, { allowDangerousXml: true }));
};
